// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'package:benchmark_harness/benchmark_harness.dart';
import 'package:ducene/ducene.dart';

class IndexWriterBenchmark extends BenchmarkBase {
  const IndexWriterBenchmark() : super("IndexWriter");

  static void main() {
    new IndexWriterBenchmark().report();
  }

  @override
  void run() {
    Analyzer ws = new WhitespaceAnalyzer(); // slow new NGramAnalyzer(2);
    IndexDirectory directory = new RAMIndexDirectory();
    IndexWriter writer = new IndexWriter(directory);
    int iteration = 10000;
    Iterable<Document> docs = new Iterable.generate(iteration, (i) {
      return new Document()
          .append('id', i.toString())
          .append('text', text, analyzer: ws);
    });
    Stopwatch stopwatch = new Stopwatch();
    stopwatch.start();
    writer.write(docs);
    print('indexed ${stopwatch.elapsed.inMilliseconds} ms');
  }
}

const String text = '''
Dart is a general-purpose programming language originally developed
by Google and later approved as a standard by Ecma (ECMA-408).[4]
It is used to build web, server and mobile applications,
and for Internet of Things (IoT) devices.[5]
It is open-source software under a BSD license.
''';

class IndexSearcherBenchmark extends BenchmarkBase {
  IndexSearcherBenchmark() : super("IndexSearcher");

  static void main() {
    new IndexSearcherBenchmark().report();
  }

  Analyzer _st;
  IndexDirectory _directory;

  @override
  void setup() {
    _st = new StandardAnalyzer();
    _directory = new RAMIndexDirectory();
    IndexWriter writer = new IndexWriter(_directory);
    int iteration = 10000;
    Iterable<Document> docs = new Iterable.generate(iteration, (i) {
      return new Document()
          .append('id', i.toString())
          .append('text', text, analyzer: _st);
    });
    Stopwatch stopwatch = new Stopwatch();
    stopwatch.start();
    writer.write(docs);
    print('indexed ${stopwatch.elapsed.inMilliseconds} ms');
  }

  @override
  void run() {
    IndexReader reader = DirectoryReader.open(_directory);
    IndexSearcher searcher = new IndexSearcher(reader);
    BoolQuery query = new BoolQuery(Op.or).append('text', 'software', analyzer: _st);
    Stopwatch stopwatch = new Stopwatch();
    stopwatch.start();
    TopDocs hits = searcher.search(query, 10);
    assert(10000 == hits.totalHits);
    print('searched ${stopwatch.elapsed.inMilliseconds} ms');
  }
}

void main() {
  IndexSearcherBenchmark.main();
  IndexWriterBenchmark.main();
}