// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'package:ducene/ducene.dart';
//import 'dart:io';

void main() {
  Analyzer st = new StandardAnalyzer();

  // Store the index in memory:
  IndexDirectory directory = new RAMIndexDirectory();
  // To store an index on disk, use this instead:
  //IndexDirectory directory = new FSIndexDirectory(new Directory("testindex"));
  IndexWriter writer = new IndexWriter(directory);
  String text = 'This is the text of #1.';
  Document doc1 = new Document()
    .append('id', '1')
    .append('text', text, analyzer: st)
    .append('filter', 'x')
    .append('cat', 'CAT-A')
    .append('subcat', 'CAT-A-1')
    .append('price', '80');
  text = 'This is the text of #2.';
  Document doc2 = new Document()
    .append('id', '2')
    .append('text', text, analyzer: st)
    .append('filter', 'x')
    .append('cat', ['CAT-B', 'CAT-A'])
    .append('subcat', 'CAT-A-2')
    .append('price', '70');
  text = "the text which 'or' matched";
  Document doc3 = new Document()
    .append('id', '3')
    .append('text', text, analyzer: st)
    .append('filter', 'x')
    .append('cat', 'CAT-C')
    .append('subcat', 'CAT-C-1')
    .append('price', '100');
  writer.write([doc1, doc2, doc3]);

  // Now search the index:
  IndexReader reader = DirectoryReader.open(directory);
  IndexSearcher searcher = new IndexSearcher(reader);
  BoolQuery query = new BoolQuery(Op.or)
      .append('text', 'this text', analyzer: st)
      .addFilter(new TermQuery(new Term('filter', 'x')));
  TopDocs hits = searcher.search(query, 1000);
  assert(3 == searcher.count(query));
  assert(3 == hits.totalHits);
  // Iterate through the results:
  for (ScoreDoc hit in hits.scoreDocs) {
    Document hitDoc = searcher.doc(hit.doc);
    assert(hitDoc.get('text').contains('the text'));
    print('${hitDoc.get('id')} : ${hit.score.toString()}');
    // Highlight:
    String snippet
      = FieldHighlight.getSnippet(query, 'text', hitDoc.get('text'));
    assert(snippet.contains('<b>'));
    print(snippet);
  }

  // Facet + Stats:
  Map<String,FacetValue> mainFacet = FieldFacet.getCount(hits.docSet,'cat',reader);
  FacetValue mainFacetA = mainFacet['CAT-A'];
  assert (mainFacetA.value == 2);
  FacetValue mainFacetB = mainFacet['CAT-B'];
  assert (mainFacetB.value == 1);
  FacetValue mainFacetC = mainFacet['CAT-C'];
  assert (mainFacetC.value == 1);
  var mainStatsA = FieldStats.getStats(mainFacetA.docSet, 'price', reader);
  assert(mainStatsA.sum == 150 && mainStatsA.mean == 75);
  var mainStatsB = FieldStats.getStats(mainFacetB.docSet, 'price', reader);
  assert(mainStatsB.count == 1);
  var mainStatsC = FieldStats.getStats(mainFacetC.docSet, 'price', reader);
  assert(mainStatsC.min == 100 && mainStatsC.max == 100);

  var subFacetA = FieldFacet.getCount(mainFacetA.docSet, 'subcat', reader);
  FacetValue subFacetA1 = subFacetA['CAT-A-1'];
  assert (subFacetA1.value == 1);
  FacetValue subFacetA2 = subFacetA['CAT-A-2'];
  assert (subFacetA2.value == 1);
  var subStatsA1 = FieldStats.getStats(subFacetA1.docSet, 'price', reader);
  assert(subStatsA1.sum == 80);
  var subStatsA2 = FieldStats.getStats(subFacetA2.docSet, 'price', reader);
  assert(subStatsA2.sum == 70);
}