// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of ducene_fb;

class FirebaseIndexDirectory implements IndexDirectory {
  FirebaseIndexStreams _indexStreams;
  FirebaseDataStreams _dataStreams;
  FirebaseIndexDirectory(this._indexStreams, this._dataStreams);
  @override
  void initOutput() { }
  @override
  void outputIndex(String name, Map<String,DocSet> index) {
    throw new Exception('UnsupportedOperationException');
  }
  @override
  void outputData(String name, Map<int,DocData> data) {
    throw new Exception('UnsupportedOperationException');
  }
  @override
  void initInput() { }
  @override
  IndexSource inputIndex() {
    return new IndexSource(_indexStreams);
  }
  @override
  DataSource inputData() {
    return new DataSource(_dataStreams);
  }
}
class FirebaseIndexStreams extends IndexStreams {
  bool _readFully;
  bool loaded = false;
  @override
  void readFully(Map to) {
    _readFully = true;
    FirebaseUtils._getFields("idx").then((fields) {
      for (String field in fields) {
        _getInvertedIndex("idx/"+field).then((invertedIndex) => to[field] = invertedIndex);
      }
    }).then((_) {loaded = true;});
  }
  Future<Map<String,DocSet>> _getInvertedIndex(String path) async{
    var invertedIndex = <String,DocSet>{};
    var query = fb.database().ref(path).orderByKey();
    var event = await query.once("value");
    event.snapshot.forEach((e) {
      String key = e.val()["term"];
      Set<int> values = (e.val()["postings"] as String).split(',').map((s) => int.parse(s)).toSet();
      invertedIndex[key] = new DocSet(values);
    });
    return invertedIndex;
  }
  @override
  void readTerms(String field, Map<String,Map<String,DocSet>> to) {
    if (!_readFully) readFully(to);
  }
  @override
  void readPostings(String field, String term, Map<String,Map<String,DocSet>> to) {
    if (!_readFully) readFully(to);
  }
}
class FirebaseDataStreams extends DataStreams {
  bool _readFully;
  bool loaded = false;
  @override
  void readFully(Map to) {
    _readFully = true;
    FirebaseUtils._getFields("data").then((fields) {
      for (String field in fields) {
        _getStoredData("data/"+field).then((storedData) => to[field] = storedData);
      }
    }).then((_) {loaded = true;});
  }
  Future<Map<int,DocData>> _getStoredData(String path) async{
    var storedData = <int,DocData>{};
    var query = fb.database().ref(path).orderByKey();
    var event = await query.once("value");
    event.snapshot.forEach((e) {
      int key = int.parse(e.key);
      List<String> values = (e.val() as String).split('\$');
      storedData[key] = new DocData(values);
    });
    return storedData;
  }
  @override
  void readFields(Map<String,Map<int,DocData>> to) {
    if (!_readFully) readFully(to);
  }
  @override
  void readValues(String field, int docId, Map<String,Map<int,DocData>> to) {
    if (!_readFully) readFully(to);
  }
}
class FirebaseUtils {
  static Future<List<String>> _getFields(String extension) async {
    List<String> fields = [];
    var query = fb.database().ref(extension).orderByKey();
    var event = await query.once("value");
    event.snapshot.forEach((e) => fields.add(e.key));
    return fields;
  }
}