// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of ducene_html;

class URLIndexDirectory implements IndexDirectory {
  final String _url;
  final List<String> _indexFields;
  final List<String> _storedFields;
  URLIndexDirectory(this._url, this._indexFields, this._storedFields);
  @override
  void initOutput() { }
  @override
  void outputIndex(String name, Map<String,DocSet> index) {
    throw new Exception('UnsupportedOperationException');
  }
  @override
  void outputData(String name, Map<int,DocData> data) {
    throw new Exception('UnsupportedOperationException');
  }
  @override
  void initInput() { }
  @override
  IndexSource inputIndex() {
    return new IndexSource(new URLIndexStreams(_indexFields, _url));
  }
  @override
  DataSource inputData() {
    return new DataSource(new URLDataStreams(_storedFields, _url));
  }

}
class URLIndexStreams extends IndexStreams {
  final List<String> _indexFields;
  final String _url;
  URLIndexStreams(this._indexFields,this._url);
  bool _readFully;
  @override
  void readFully(Map to) {
    _readFully = true;
    //for (String field in _indexFields) {
    Future.forEach(_indexFields, (field) {
      URLUtils._get(_url + '/$field.idx').then((String str) {
        List<String> lines = str.split('\n');
        var invertedIndex = <String,DocSet>{};
        for (String line in lines) {
          List<String> kv = line.split('=>');
          if (kv.length == 2) {
            String key = kv[0];
            Set<int> values = kv[1].split(',').map((s) => int.parse(s)).toSet();
            invertedIndex[key] = new DocSet(values);
          }
        }
        to[field] = invertedIndex;
      });
    });//}
  }
  @override
  void readTerms(String field, Map<String,Map<String,DocSet>> to) {
    if (!_readFully) readFully(to);
  }
  @override
  void readPostings(String field, String term, Map<String,Map<String,DocSet>> to) {
    if (!_readFully) readFully(to);
  }
}
class URLDataStreams extends DataStreams {
  final List<String> _storedFields;
  final String _url;
  URLDataStreams(this._storedFields,this._url);
  bool _readFully;
  @override
  void readFully(Map to) {
    _readFully = true;
    //for (String field in _storedFields) {
    Future.forEach(_storedFields, (field) {
      URLUtils._get(_url + '/$field.data').then((String str) {
        List<String> lines = str.split('\n');
        var storedData = <int,DocData>{};
        for (String line in lines) {
          List<String> kv = line.split('=>');
          if (kv.length == 2) {
            int key = int.parse(kv[0]);
            List<String> values = kv[1].split('\$');
            storedData[key] = new DocData(values);
          }
        }
        to[field] = storedData;
      });
    });//}
  }
  @override
  void readFields(Map<String,Map<int,DocData>> to) {
    if (!_readFully) readFully(to);
  }
  @override
  void readValues(String field, int docId, Map<String,Map<int,DocData>> to) {
    if (!_readFully) readFully(to);
  }
}
class URLUtils {
  static Future _get(String url) async {
    return html.HttpRequest.getString(url);
  }
}