// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'package:ducene/ducene.dart';
import 'package:test/test.dart';

void main() {
  group('A group of searcher', () {

    test('should have correct counts', () {
      IndexDirectory indexDirectory = new RAMIndexDirectory();
      IndexWriter writer = new IndexWriter(indexDirectory);
      List<Document> docs = [];
      for (int i = 0; i < 10; i++) {
        Document doc = new Document();
        doc.append("id", i.toString());
        doc.append("all", "x");
        doc.append("mv", ["m", "v"]);
        doc.append("filter", "filter");
        doc.append("price1", (i + 1).toString());
        doc.append("price2", ((i + 1)*100).toString());
        docs.add(doc);
      }
      writer.write(docs);

      IndexReader reader = DirectoryReader.open(indexDirectory);
      IndexSearcher searcher = new IndexSearcher(reader);

      expect(searcher.count(new BoolQuery(Op.and).append('id', '5')), 1);
      expect(searcher.count(new BoolQuery(Op.and).append('all', 'x')), 10);
      expect(searcher.count(new BoolQuery(Op.and).append('id', 'dummy')), 0);
      expect(searcher.count(new BoolQuery(Op.and).append('all', 'dummy')), 0);
      expect(searcher.count(new BoolQuery(Op.or).append('id', '5')), 1);
      expect(searcher.count(new BoolQuery(Op.or).append('all', 'x')), 10);
      expect(searcher.count(new BoolQuery(Op.or).append('id', 'dummy')), 0);
      expect(searcher.count(new BoolQuery(Op.or).append('all', 'dummy')), 0);

      expect(searcher.count(new BoolQuery(Op.or).append('mv', 'm')), 10);
      expect(searcher.count(new BoolQuery(Op.or).append('mv', 'v')), 10);

      BoolQuery q = new BoolQuery(Op.or)
          .append('all', 'x')
          .addFilter(new TermQuery(new Term('filter', 'filter')));
      expect(searcher.count(q), 10);
      q = new BoolQuery(Op.or)
          .append('all', 'x')
          .addFilter(new TermQuery(new Term('filter', 'dummy')));
      expect(searcher.count(q), 0);
      q = new BoolQuery(Op.or)
          .append('all', 'x')
          .addFilter(new TermQuery(new Term('filter', 'filter')))
          .addFilter(new TermQuery(new Term('filter', 'dummy')));
      expect(searcher.count(q), 0);
      q = new BoolQuery(Op.or)
          .append('all', 'x')
          .addFilter(new TermQuery(new Term('filter', 'filter')))
          .addFilter(new RangeQuery('price1', 3.00, 5.00));
      expect(searcher.count(q), 3);
      q = new BoolQuery(Op.or)
          .append('all', 'x')
          .addFilter(new TermQuery(new Term('filter', 'filter')))
          .addFilter(new RangeQuery('price1', 3.00, 5.00))
          .addFilter(new RangeQuery('price2', 400.00, 500.00));
      expect(searcher.count(q), 2);
    });

    test('synonym filtering and not filtering', () {
      IndexDirectory indexDirectory = new RAMIndexDirectory();
      IndexWriter writer = new IndexWriter(indexDirectory);
      Document doc1 = new Document()
        ..append("id", '01')
        ..append("all", "x")
        ..append("text", "a");
      Document doc2 = new Document()
        ..append("id", '02')
        ..append("all", "x")
        ..append("text", "b");
      Document doc3 = new Document()
        ..append("id", '03')
        ..append("all", "x")
        ..append("text", "c");
      writer.write([doc1, doc2, doc3]);
      IndexReader reader = DirectoryReader.open(indexDirectory);
      IndexSearcher searcher = new IndexSearcher(reader);
      BoolQuery q = new BoolQuery(Op.or)
          .append('all', 'x')
          .addFilter(new TermsQuery([new Term('text', 'a')]));
      expect(searcher.count(q), 1);
      q = new BoolQuery(Op.or)
          .append('all', 'x')
          .addFilter(new TermsQuery([new Term('text', 'a'),new Term('text', 'c')]));
      expect(searcher.count(q), 2);

      Term allTerm = new Term('all', 'x');
      q = new BoolQuery(Op.or)
          .addQuery(new TermQuery(allTerm))
          .addFilter(new NotQuery(allTerm, [new Term('text', 'a')]));
      expect(searcher.count(q), 2);
      q = new BoolQuery(Op.or)
          .addQuery(new TermQuery(allTerm))
          .addFilter(new NotQuery(allTerm, [new Term('text', 'a'),new Term('text', 'c')]));
      expect(searcher.count(q), 1);
      /*for (ScoreDoc doc in searcher.search(q,3,new MatchScoreSort()).scoreDocs) {
        print(searcher.doc(doc.doc).get("id"));
      }*/

    });

  });
}

