// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'package:ducene/ducene.dart';
import 'package:test/test.dart';

void main() {
  group('A group of writer', () {

    test('test rewrite', () {
      IndexDirectory indexDirectory = new RAMIndexDirectory();
      IndexWriter writer = new IndexWriter(indexDirectory);
      List<Document> docs = [];
      for (int i = 0; i < 10; i++) {
        Document doc = new Document();
        doc.append("id", i.toString());
        docs.add(doc);
      }
      writer.write(docs);

      expect(DirectoryReader.open(indexDirectory).maxDoc(), 10);

      docs = [];
      for (int i = 0; i < 3; i++) {
        Document doc = new Document();
        doc.append("id", i.toString());
        docs.add(doc);
      }
      writer.write(docs);

      expect(DirectoryReader.open(indexDirectory).maxDoc(), 3);
    });

    test('test index and search get()', () {
      IndexDirectory indexDirectory = new RAMIndexDirectory();
      IndexWriter writer = new IndexWriter(indexDirectory);
      Document doc = new Document();
      doc.append("id", '1');
      doc.append("text1", 'text1');
      doc.append("text2", ['text2-1', 'text2-2']);
      doc.append("text3", 'text3', stored: false);
      doc.append("text4", ['text4-1', 'text4-2'], stored: false);
      writer.write([doc]);

      IndexReader reader = DirectoryReader.open(indexDirectory);
      IndexSearcher searcher = new IndexSearcher(reader);
      TopDocs docs = searcher.search(new BoolQuery(Op.or).append("id", '1'), 10, scoreSort: ScoreSort.indexed());
      expect(docs.totalHits, 1);
      ScoreDoc scoreDoc = docs.scoreDocs[0];
      Document d = searcher.doc(scoreDoc.doc);
      expect(d.get("id"), '1');
      expect(d.get("text1"), 'text1');
      expect(d.getValues("text2"), ['text2-1', 'text2-2']);
      expect(d.get("text3"), isNull);
      expect(d.getValues("text4"), isNull);
    });

  });
}

